@@ -12,6 +12,12 @@ class DebugControl(object):
         self._ctrl = controlobj
         exception.wrap_comclass(self._ctrl)
 
+    def Release(self):
+        cnt = self._ctrl.Release()
+        if cnt == 0:
+            self._ctrl = None
+        return cnt
+
     # IDebugControl
 
     def GetInterrupt(self):
@@ -149,9 +155,9 @@ class DebugControl(object):
         raise exception.E_NOTIMPL_Error
 
     def GetStackTrace(self):
-        frames = (DbgEng._DEBUG_STACK_FRAME * 50)()
+        frames = (DbgEng._DEBUG_STACK_FRAME * 100)()
         count = c_ulong()
-        hr = self._ctrl.GetStackTrace(0, 0, 0, frames, 50, byref(count))
+        hr = self._ctrl.GetStackTrace(0, 0, 0, frames, 100, byref(count))
         exception.check_err(hr)
         return list(frames)[:count.value]
 
