@@ -1,3 +1,4 @@
+import io
 import itertools
 import queue
 import struct
@@ -33,7 +34,7 @@ def InitComObjects(Dbg):
     Dbg.mod             = Modules(Dbg._dataspaces, Dbg._symbols)
     Dbg.events          = EventHandler(Dbg)
     Dbg.breakpoints     = Breakpoints(Dbg._control)
-    Dbg.callbacks       = DbgEngCallbacks(Dbg.events, sys.stdout.write)
+    Dbg.callbacks       = DbgEngCallbacks(Dbg.events, sys.stdout)
 
     Dbg.events.breakpoint(Dbg.breakpoints)
 
@@ -47,12 +48,48 @@ def InitComObjects(Dbg):
     Dbg._client.SetEventCallbacks(Dbg.callbacks)
 
 
+def FiniComObjects(Dbg):
+    #Dbg.breakpoints.remove_all()
+    Dbg._client.SetOutputCallbacks(None)
+    Dbg._client.SetEventCallbacks(None)
+    Dbg.callbacks = None
+    Dbg.reg = None
+    Dbg.mod = None
+    Dbg.events = None
+    Dbg.breakpoints = None
+
+    """
+    x = Dbg._advanced.Release()
+    print(f"{x=}")
+    x = Dbg._control.Release()
+    print(f"{x=}")
+    x = Dbg._registers.Release()
+    print(f"{x=}")
+    x = Dbg._dataspaces.Release()
+    print(f"{x=}")
+    x = Dbg._symbols.Release()
+    print(f"{x=}")
+    x = Dbg._systems.Release()
+    print(f"{x=}")
+    x = Dbg._client.Release()
+    print(f"{x=}")
+    """
+    Dbg._advanced = None
+    Dbg._control = None
+    Dbg._registers = None
+    Dbg._dataspaces = None
+    Dbg._symbols = None
+    Dbg._systems = None
+    Dbg._client = None
+
+
 def EventThread(Dbg, Ev, WorkQ):
     Dbg._client = DebugClient()
     InitComObjects(Dbg)
     Ev.set()
+    run = True
 
-    while True:
+    while run:
         item = WorkQ.get(True)
         if item.task == 'WaitForEvent':
             try:
@@ -75,6 +112,9 @@ def EventThread(Dbg, Ev, WorkQ):
             except Exception as ex:
                 #print("DispatchCallbacks exception", ex)
                 pass
+        elif item.task == 'Quit':
+            run = False
+
         WorkQ.task_done()
         Ev.set()
 
@@ -101,6 +141,14 @@ class DebuggerBase(object):
 
             InitComObjects(self)
 
+    def Release(self):
+        if self.standalone:
+            self._worker_wait("Quit")
+            self._thread = None
+        else:
+            pass
+        #FiniComObjects(self)
+
     def _reset_callbacks(self):
         self._client.SetEventCallbacks(self.callbacks)
 
@@ -238,9 +286,17 @@ class DebuggerBase(object):
         else:
             self._control.WaitForEvent(timeout)
 
-    def cmd(self, cmdline):
+    def cmd(self, cmdline, quiet=True):
         """cmd(cmdline) -> execute a windbg console command"""
+        buffer = io.StringIO()
+        self.callbacks.stdout = buffer
         self._control.Execute(cmdline)
+        self.callbacks.reset_stdout()
+        buffer.seek(0)
+        data = buffer.read()
+        if not quiet:
+            print(data)
+        return data
 
     def bitness(self):
         """bitness() -> Return target bitness"""
@@ -253,6 +309,24 @@ class DebuggerBase(object):
         """read(addr,len) -> read len bytes from addr"""
         return self._dataspaces.ReadVirtual(addr, len)
 
+    def readstr(self, addr, wchar=False):
+        """readstr(addr, wchar=False) -> return string"""
+        MAX_LEN = 0x1000
+        if wchar:
+            width = 2
+            enc = 'utf-16-le'
+        else:
+            width = 1
+            enc = 'utf-8'
+        data = []
+        for i in range(0, MAX_LEN, width):
+            x = self.read(addr + i, width)
+            if x == b"\x00"*width:
+                break
+            else:
+                data.append(x)
+        return b''.join(data).decode(enc)
+
     def readptr(self, addr, count=1):
         """readptr(addr,count=1) -> read and return size_t dwords from addr"""
         if self.bitness() == '64':
@@ -264,6 +338,10 @@ class DebuggerBase(object):
         data = self.read(addr, count * width)
         return struct.unpack("<" + fmt*count, data)
 
+    def poi(self, addr):
+        """poi(addr) -> read pointer from addr"""
+        return self.readptr(addr)[0]
+
     def write(self, addr, data):
         """write(addr,data) -> write data to addr"""
         return self._dataspaces.WriteVirtual(addr, data)
@@ -356,7 +434,10 @@ class DebuggerBase(object):
 
     def symbol(self, name):
         """symbol(name) -> resolve a symbol"""
-        return self._symbols.GetOffsetByName(name)[1]
+        try:
+            return self._symbols.GetOffsetByName(name)[1]
+        except exception.E_FAIL_Error:
+            return -1
 
     def find_symbol(self, pattern):
         """symbol(name) -> resolve a symbol"""
@@ -405,21 +486,7 @@ class DebuggerBase(object):
 
     def ds(self, addr, wchar=False):
         """ds(addr, wchar=False) -> display string"""
-        MAX_LEN = 0x1000
-        if wchar:
-            width = 2
-            enc = 'utf-16-le'
-        else:
-            width = 1
-            enc = 'ascii'
-        data = []
-        for i in range(0, MAX_LEN, width):
-            x = self.read(addr + i, width)
-            if x == "\x00"*width:
-                break
-            else:
-                data.append(x)
-        print(''.join(data).decode(enc))
+        print(self.readstr(addr,wchar))
 
     def bl(self):
         """bl() -> List breakpoints"""
